import json
import os
import operator as op
import pathlib
import sys
from zipfile import ZipFile

from cryptography.fernet import Fernet
from pathlib import Path
from loguru import logger

logger.configure(**{"handlers": [{"sink": sys.stdout, "format": "<green>{time:YYYY-MM-DD HH:mm:ss}</green> | <level>{level: <8}</level> | <cyan><level>{message}</level></cyan>"}]})


def getCWD_Root() -> Path:
    """
    Returns the project root directory for the current project we're in.

    :return: path to the root of the project
    :rtype: string
    """
    return Path(os.getcwd())


def getPackage_Root() -> Path:
    return os.path.dirname(os.path.abspath(__file__))


def getProject_Root() -> Path:
    # return Path(__file__).parent.parent
    return Path(os.getcwd())


class pkJson:
    def __init__(self):
        self.ferKey = None
        self.ferKeyFile = Path(getProject_Root()) / ".pkJsonrc"
        self.pkJsonFile = Path(getProject_Root()) / "pkJson.json"
        self.getKey()

    def getKey(self):

        if os.path.exists(self.ferKeyFile):
            with open(self.ferKeyFile, 'rb') as f:
                self.ferKey = f.read()
        else:
            self.ferKey = Fernet.generate_key()

            with open(self.ferKeyFile, 'wb') as f:
                f.write(self.ferKey)

    def encryptPath(self, filepath):
        fernet = Fernet(self.ferKey)
        filepath = self.isString(filepath)  # if str convert to byte
        encrypted = fernet.encrypt(filepath)

        return encrypted.decode()

    def decryptPath(self, filepath):
        fernet = Fernet(self.ferKey)
        filepath = self.isString(filepath)  # if str convert to byte
        decrypted = fernet.decrypt(filepath)

        return decrypted

    def encryptFile(self, filename):
        fernet = Fernet(self.ferKey)

        with open(filename, 'rb') as file:
            original = file.read()

        encrypted = fernet.encrypt(original)

        return encrypted.decode()

    def decryptContent(self, content):
        fernet = Fernet(self.ferKey)
        content = self.isString(content)  # if str convert to byte
        decrypted = fernet.decrypt(content)

        return decrypted

    def createFileFolder(self, fullpath, content, jsonProject=".jsonProject"):
        fullpath = self.decryptPath(fullpath).decode()
        filename = os.path.basename(fullpath)
        filepath = os.path.dirname(fullpath)

        oFilePath = Path(getProject_Root()) / jsonProject / filepath
        oFileFull = Path(getProject_Root()) / jsonProject / filepath / filename
        oFileCreating = Path(jsonProject) / filepath / filename

        logger.info(f'Creating: {oFileCreating}')

        if not os.path.exists(oFilePath):
            os.makedirs(oFilePath)

        try:
            # logger.error(oFileFull)
            with open(f'{oFileFull}', 'wb') as f:
                f.write(self.decryptContent(content))
        except Exception:
            logger.error(f'Error creating file: {jsonProject}')

    def encryptProject(self, projectDir):
        self.getKey()

        desktop = pathlib.Path(projectDir)
        # desktop = pathlib.Path(getProject_Root()) / projectDir

        logger.warning(f'{desktop}')

        desktop.rglob("*")

        project = {self.encryptPath("ferKey"): self.ferKey.decode()}
        for o in list(desktop.rglob("*")):
            if not self.ifExists(str(o)) and not o.is_dir():
                fullPath = str(o)
                logger.info(f'Adding: {fullPath}')
                key = self.encryptPath(fullPath)
                value = self.encryptFile(fullPath)
                project[key] = value
        # projectDir = 'jsonProject'

        with open(f'{self.pkJsonFile}', 'wb') as encrypted_file:
            encrypted_file.write(json.dumps(project, indent=2).encode())

        # with ZipFile('pkJson.zip', 'w') as oZip:
        #     oZip.write(self.pkJsonFile)

    def decryptProject(self, projectJson):
        oJson = {}
        if os.path.exists(projectJson):
            with open(projectJson) as f:
                oJson = json.load(f)
        else:
            logger.error(f'{projectJson} not found...')

        cnt = 0
        for k, v in oJson.items():
            if cnt == 0:
                self.ferKey = v
            else:
                self.createFileFolder(k, v)
            cnt += 1

    @staticmethod
    def ifExists(string):
        res = False
        test_list = ['venv', '.git', '__pycache__', '.idea', 'seCodeCoverage', 'jsonProject', '.pytest_cache',
                     'egg-info', '.pkJsonrc', '.DS_Store', '.coverage', '.zip', 'pkJson.json', 'tar.gz', 'any.whl']
        for i in test_list:
            if op.contains(string, i):
                res = op.contains(string, i)
                break

        return res

    @staticmethod
    def isString(obj):
        if isinstance(obj, str):
            obj = obj.encode()
        return obj


if __name__ == '__main__':
    oJsonProject = pkJson()

    # rootDir = '.'
    # oJsonProject.encryptProject(rootDir)

    oJsonProject.decryptProject("jsonProject.json")



import json
import os
import re

from pydantic_settings import BaseSettings
from pkJson._utils import getProject_Root

# Project Directories
ROOT = getProject_Root()

settingDefault = '''# Configuration Settings
# ----------------------------------------------------------------------------------------------------------------------
# NOTE: No need to use quotes around values but you can use single, double or no quotes.
#       The values are the items to the right of the equals sign, and comments are striped from line.
# ----------------------------------------------------------------------------------------------------------------------

[Logging]
log_level               = 'DEBUG'                       # Set logging level. Options: DEBUG, INFO, WARNING
log_appname             = 'pkJson'                      # Used to write to standard out what app is logging
log_inc_resp_body       = 'f'                           # Valid: 'true', 't', '1', 'false', 'f', '0'

[Settings]
setting_1               = 'value_1'
setting_2               = 'value_2'
'''


class ConfigManager:
    """
    Class manages loading the configuration parameters from .env.settings and .env.secrets
    """

    def __init__(self, pramCheck=None):
        if pramCheck is None:
            pramCheck = dict()
        self.parameters = {}
        self.pramCheck = pramCheck
        self.error_list = {}
        self.error_occurred = False

        # -------------------------------------------------------------------------------------------------------------------------------------------
        # create default .env file if not exist, then load
        # -------------------------------------------------------------------------------------------------------------------------------------------
        filepath = f'{ROOT}/.pkJsonrc'
        print(filepath)
        if not os.path.exists(filepath):

            f = open(filepath, "w")
            f.write(settingDefault)
            f.close()

        if os.path.exists(filepath):
            self.readFile(filepath)
        else:
            print(filepath)

    def readFile(self, filename):
        with open(filename, "r") as fEnv:
            lines = fEnv.readlines()
            count = 0
            search_pattern = re.compile(r'^[^#].+=[\"|\']*.*[\"|\']*$')

            for line in lines:
                count += 1
                if search_pattern.search(line):
                    # remove line comments
                    key_value_comment = line.strip().split("#", 1)

                    # split parameters/value
                    name, value = key_value_comment[0].strip().split("=", 1)

                    # remove single or double quotes from value
                    value = value.translate(str.maketrans('', '', '"\''))

                    # Remove all spaces on both sides of each object
                    name = name.strip()
                    value = value.strip()
                    self.parameters[name] = value

    def validateKeyList(self):

        if self.pramCheck:
            for parm in self.pramCheck:
                # The self.parameterList[parameters] element contains the keywords to determine whether a parameter is
                # required or optional.  If it is required, it should have the value "ERROR".  If it's optional,
                # it can be something like WARNING, INFO, or whatever else fits the purpose
                if not parm in self.parameters:
                    self.error_list[parm] = json.dumps({"settingsRequired": {"details": f'Setting \'{parm}\' does not exist in the config file and is required', "level": f'{self.pramCheck[parm]}'}})
                elif not self.parameters[parm]:
                    self.error_list[parm] = json.dumps({"settingsRequired": {"details": f'Setting \'{parm}\' is in the config file but is not set', "level": f'WARNING'}})

        self.error_occurred = self.checkConfigFileParameters()

    def checkConfigFileParameters(self):
        # This function checks the error_list array that is created in the init of this class
        # error_occurred = [e for e,v in self.error_list if re.search('ERROR',v)]
        error_occurred = False
        for (e, v) in self.error_list.items():
            if re.search('ERROR', v, re.IGNORECASE):
                error_occurred = True
        return error_occurred


settingsValidate = {"log_level": "ERROR", "log_appname": "ERROR", "log_inc_resp_body": "ERROR"}
oConfigManager = ConfigManager(settingsValidate)
oConfigManager.validateKeyList()


class Settings(BaseSettings):
    SETTING_ERRORS: dict = {}
    FN_KEY: str = os.getenv('FN_KEY', 'l911qB1keWvIykhvswzdKCQbr6h35Cabu8OeckOUbP4=')

    # Settings from .pkJsonrc
    try:
        LOG_LEVEL: str = oConfigManager.parameters['log_level']
        LOG_APPNAME: str = oConfigManager.parameters['log_appname']
        SETTING_1: str = oConfigManager.parameters['setting_1']
        SETTING_2: str = oConfigManager.parameters['setting_2']

    except KeyError as ex:
        SETTING_ERRORS[ex.args[0]] = json.dumps({"settingsErrors": {"details": f'Setting {ex} does not exist in config.Settings. Needs to be removed if not required'}})

    @staticmethod
    def validateSettings():
        missing = {}
        for param in oConfigManager.parameters:
            try:
                eval(f'settings.{param.upper()}')

            except AttributeError:
                missing[param] = json.dumps({"settingsMissing": {"details": f'{param.upper()}: str = oConfigManager.parameters[\'{param}\']'}})

        return missing


settings = Settings()

settingsMissing = settings.validateSettings()
settingsErrors = settings.SETTING_ERRORS
settingsNotSet = oConfigManager.error_list

for k, v in sorted(settings):
    print(f'{k:>25}: {v}')

if len(settingsMissing):
    print("Need to add to pkJson._settings:")
    for k, v in settingsMissing.items():
        o = json.loads(v)
        print(f'{o["settingsMissing"]["details"]}')



"""
Provides `pkJson` version information.
"""
# This file is auto-generated! Do not edit!
# Use `seVersion pkJson` to change this file.

from seVersion import Version
__version__ = Version("pkJson", 2024, 1, 0, 0, release_candidate=2)
__setuptools__ = __version__.public()
__all__ = ["__version__", "__setuptools__"]



import sys

import click
from loguru import logger
from pkJson._utils import pkJson, getCWD_Root, getPackage_Root, getProject_Root

logger.configure(**{"handlers": [{"sink": sys.stdout, "format": "<green>{time:YYYY-MM-DD HH:mm:ss}</green> | <level>{level: <8}</level> | <cyan><level>{message}</level></cyan>"}]})


oPk = pkJson()


# def getCWD_Root() -> Path:
#     """
#     Returns the project root directory for the current project we're in.
#
#     :return: path to the root of the project
#     :rtype: string
#     """
#     return Path(os.getcwd())
#
#
# def getPackage_Root() -> Path:
#     return os.path.dirname(os.path.abspath(__file__))
#
#
# def getProject_Root() -> Path:
#     return Path(__file__).parent.parent


def _version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(f'{__version__}')
    ctx.exit()


@click.group()
@click.option('-v', '--version', is_flag=True, callback=_version, expose_value=False, is_eager=True)
def cli():
    pass


@cli.command()
def settings():
    """Check configuration settings"""
    logger.info("pkJson: Config Settings")
    logger.info(f'{"":->160}')
    logger.info(f'CDW......: {getCWD_Root()}')
    logger.info(f'Package..: {getPackage_Root()}')
    logger.info(f'Project..: {getProject_Root()}')
    logger.info(f'Key File.: {oPk.ferKeyFile} --> {oPk.ferKey.decode()}')
    logger.info(f'Json File: {oPk.pkJsonFile}')


@cli.command()
@click.argument('path', default='.')
def encode(path):
    """Encode project into Json file"""

    oPk.encryptProject(path)


@cli.command()
@click.argument('filename', default='pkJson.json')
def decode(filename):
    """Encode project into Json file"""

    oPk.decryptProject(filename)
